// OV9640 Camera_Module Library for EBM300 by NKJ.  2010.01


////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// SCCB /////////////////////////////////////// 

#define cm_clock(X)  PORTD.6=X
void cm_data_wr(unsigned char x)
{
  DDRD.7=1; 
  PORTD.7=x; 
}
unsigned char cm_data_rd(void)
{
  DDRD.7=0;  
  return PIND.7;
}

void sccb_delay(void)
{
  unsigned int d=50;  // about 50uS...14.7456MHz 
  while(d--)
   { 
      #asm("nop")
      #asm("nop")
      #asm("nop")
      #asm("nop")
   }   
}

void sccb_init(void)
{
  cm_data_wr(1);
  cm_clock(1);	
}

void sccb_start(void)  // Start of transmission
{
  cm_data_wr(0);
  sccb_delay();	
  cm_clock(0);
}

void sccb_stop(void)   // Stop of transmission
{
  sccb_delay();
  cm_data_wr(0);	
  cm_clock(1);
  sccb_delay();	
  cm_data_wr(1);
}

void sccb_write(unsigned char ddata)
{ 
  unsigned char n;
   
  for(n=0;n<8;n++) 
   {
     sccb_delay();	   

     if( (ddata<<n)&0x80 ) cm_data_wr(1); 
     else cm_data_wr(0);  

     cm_clock(1);
     sccb_delay();
     cm_clock(0);
   }
   
  cm_data_rd();  // for 9th bit.  
  sccb_delay();
  cm_clock(1);
  sccb_delay();
  cm_clock(0);
}


unsigned char sccb_read(void)
{ 
  unsigned char n;
  unsigned char ddata=0;

  for(n=9;n>1;n--)
   {
     sccb_delay();
     cm_clock(1);
     sccb_delay();

     ddata |= cm_data_rd() << (n-2);

     cm_clock(0);
   }

  sccb_delay();
  cm_data_wr(1);   // for read state	 
  cm_clock(1);
  sccb_delay();
  cm_clock(0);
   
  return ddata;
}

void SetOV9640(unsigned char addr,unsigned char ddata)
{
  sccb_start();
  sccb_write(0x60);  // slave device address
  sccb_write(addr);
  sccb_write(ddata);
  sccb_stop();

  mdelay(5);
}


void OV9640RegInit(void)  
{ 
  SetOV9640(0x00,0x32);
  SetOV9640(0x01,0x5a);
  SetOV9640(0x02,0x33);
  SetOV9640(0x09,0x00);
//  SetOV9640(0x14,0x4a)
  SetOV9640(0x14,0x8a);  // Automactic Gain 8x
  SetOV9640(0x15,0x00);
  SetOV9640(0x16,0x00);
  SetOV9640(0x1B,0x01);
  SetOV9640(0x1C,0x7F);
  SetOV9640(0x1D,0xA2);
  SetOV9640(0x1E,0x00);
  SetOV9640(0x1F,0x00);
  SetOV9640(0x27,0xB0);
  SetOV9640(0x28,0x90);
  SetOV9640(0x2C,0x10);
  SetOV9640(0x2D,0x00);
  SetOV9640(0x2E,0x00);
  SetOV9640(0x33,0x08);
  SetOV9640(0x34,0x03);
//  SetOV9640(0x37,0x0C);
  SetOV9640(0x37,0x04);  // ADC range adjustment 1x
  SetOV9640(0x38,0x92);  // 2x Gain for analog
                            
  SetOV9640(0x3A,0x00);       
  SetOV9640(0x3C,0x46);
//  SetOV9640(0x3D,0x98);       
  SetOV9640(0x3D,0x80);         // for RGB565
  SetOV9640(0x3E,0x02);     
  SetOV9640(0x3F,0xF2);
//  SetOV9640(0x41,0x02);
  SetOV9640(0x41,0x10); // default
  SetOV9640(0x42,0xC8);     
  SetOV9640(0x43,0xF0);     
  SetOV9640(0x44,0x10);
  SetOV9640(0x45,0x5C);
  SetOV9640(0x46,0x70);
  SetOV9640(0x47,0x46);
  SetOV9640(0x48,0x4D);
  SetOV9640(0x49,0x00);
  SetOV9640(0x4a,0x00);
  SetOV9640(0x4b,0x00);
  SetOV9640(0x4c,0x00);
  SetOV9640(0x4d,0x00);
  SetOV9640(0x4e,0x00);
  SetOV9640(0x4f,0x47);
  SetOV9640(0x50,0x38);       
  SetOV9640(0x51,0x0F); 
  SetOV9640(0x52,0x10);
  SetOV9640(0x53,0x6B); 
  SetOV9640(0x54,0x7B);
  SetOV9640(0x55,0x40);
  SetOV9640(0x56,0x40);
  SetOV9640(0x57,0x40);
  SetOV9640(0x58,0x0F);
  SetOV9640(0x59,0xCF);
  SetOV9640(0x5A,0xe7);
  SetOV9640(0x5B,0x57);
  SetOV9640(0x5C,0x86);
  SetOV9640(0x5D,0x5f);
  SetOV9640(0x5E,0x0A);
  SetOV9640(0x5F,0x05); 
  SetOV9640(0x60,0x14);
  SetOV9640(0x61,0xCE);
  SetOV9640(0x62,0x00);
  SetOV9640(0x63,0x00);      
  SetOV9640(0x64,0x03);
  SetOV9640(0x65,0x10);	
  SetOV9640(0x66,0x01);
  SetOV9640(0x67,0x80);
  SetOV9640(0x68,0x80);
  SetOV9640(0x69,0x00);
  SetOV9640(0x6A,0x00);
  SetOV9640(0x6B,0x3f);
  SetOV9640(0x6C,0x40);
  SetOV9640(0x6D,0x30);
  SetOV9640(0x6E,0x4B);
  SetOV9640(0x6F,0x60);
  SetOV9640(0x70,0x70);
  SetOV9640(0x71,0x70);
  SetOV9640(0x72,0x70);
  SetOV9640(0x73,0x70);      
  SetOV9640(0x74,0x60);
  SetOV9640(0x75,0x60);
  SetOV9640(0x76,0x50);
  SetOV9640(0x77,0x48);
  SetOV9640(0x78,0x3A);
  SetOV9640(0x79,0x2E);
  SetOV9640(0x7A,0x28);
  SetOV9640(0x7B,0x22);
  SetOV9640(0x7C,0x04);
  SetOV9640(0x7D,0x07);
  SetOV9640(0x7E,0x10);
  SetOV9640(0x7F,0x28);
  SetOV9640(0x80,0x36);
  SetOV9640(0x81,0x44);
  SetOV9640(0x82,0x52);
  SetOV9640(0x83,0x60);
  SetOV9640(0x84,0x6c);
  SetOV9640(0x85,0x78);
  SetOV9640(0x86,0x8c);
  SetOV9640(0x87,0x9e);
  SetOV9640(0x88,0xbb);
  SetOV9640(0x89,0xd2);
  SetOV9640(0x8a,0xe6);	     
}

unsigned char AEW[5]={ 0x40, 0x50, 0x80, 0x90, 0xA0 };
unsigned char AEB[5]={ 0x30, 0x40, 0x70, 0x80, 0x90 };
unsigned char VPT[5]={ 0x80, 0x91, 0xA4, 0xB5, 0xC6 };

void EnableAutoSetCM(void) 
{
  sccb_init();
  CmReset(1);
  mdelay(50);
  CmReset(0);
  mdelay(50);

  SetOV9640(0x12,0x80);      // Reset Register 
  SetOV9640(0x12,0x00);   

  OV9640RegInit();       
  
  SetOV9640(0x0c,0x04);      // two pixel average.
  SetOV9640(0x0d,0xc0);   
  SetOV9640(0x3C,0x40);          
}


void EnableCM(void) 
{
  sccb_init();
  CmReset(1);
  mdelay(50);
  CmReset(0);
  mdelay(50);

//SXGA(1280*960) RawRGB Setting 
/*  SetOV9640(0x12,0x80);      // Reset Register 
  SetOV9640(0x12,0x00);   
  SetOV9640(0x12,0x05);      // RAW RGB Selection
*/

//QVGA(320*240) RawRGB Setting 
  SetOV9640(0x12,0x80);      // Reset Register 
  SetOV9640(0x12,0x00);   
  SetOV9640(0x12,0x15);      // RAW RGB Selection

//  SetOV9640(0x12,0x04);      // RGB Selection
//  SetOV9640(0x40,0xd0);      // RGB565 Selection
//  SetOV9640(0x11,0x1f);      // pre-scalar
//  SetOV9640(0x11,0x80);      // double clock..

  SetOV9640(0x13,0x00);     // Disable AGC/AEC/AWB

  OV9640RegInit();

  SetOV9640(0x24,AEW[2]);
  SetOV9640(0x25,AEB[2]);
  SetOV9640(0x26,VPT[2]);
  
/* // My Setting
  SetOV9640(0x4f,0xf0);  
  SetOV9640(0x50,0xf0); 
  SetOV9640(0x51,0x0f); 
  SetOV9640(0x52,0x10);
  SetOV9640(0x53,0x6B); 
  SetOV9640(0x54,0x7B);
  SetOV9640(0x55,0x80);
  SetOV9640(0x56,0x73);
  SetOV9640(0x57,0x10);

  SetOV9640(0x58,0xcf);
*/
}


void DisableCM(void) 
{
  CmReset(1);
}

void SetRawDataSXGA(void)
{                       
  SetOV9640(0x12,0x05);      // RAW RGB SXGA(1280*960) Selection
  SetOV9640(0x40,0x00);     
  SetOV9640(0x04,0x00);      // 
}
void SetRawDataVGA(void)
{                           
  SetOV9640(0x12,0x45);      // RAW RGB VGA(640*480) Selection
  SetOV9640(0x40,0x00); 
  SetOV9640(0x04,0x00);      // 
}

void SetRawDataQVGA(void)
{
  SetOV9640(0x12,0x15);      // RAW RGB QVGA(320*240) Selection
  SetOV9640(0x40,0x00); 
  SetOV9640(0x04,0x00);      //   
}         

void SetRawDataQQVGA(void)
{
  SetOV9640(0x12,0x15);      // RAW RGB QVGA Selection
  SetOV9640(0x40,0x00); 
  SetOV9640(0x04,0x20);      // Set QQVGA(160*120)   
}    
 

void SetRGB565VGA(void)
{
  SetOV9640(0x12,0x44);      // VGA,RGB 
  SetOV9640(0x40,0xd0);      // RGB565 QVGA(640*840) Selection    
  SetOV9640(0x04,0x00);      //   
}

void SetRGB565QVGA(void)
{
  SetOV9640(0x12,0x14);      // QVGA,RGB 
  SetOV9640(0x40,0xd0);      // RGB565 QVGA(320*240) Selection    
  SetOV9640(0x04,0x00);      // 
}

void SetRGB565QQVGA(void)
{
  SetOV9640(0x12,0x14);      // QVGA,RGB 
  SetOV9640(0x40,0xd0);      // RGB565 QVGA Selection    
  SetOV9640(0x04,0x20);      // Set QQVGA(160*120)
}

/////////////////////////////////////// SCCB /////////////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////////

 